@@ -10,7 +10,6 @@
 #include "ifopt/ifoptutil.h"
 
 #include <KPublicTransport/Journey>
-#include <KPublicTransport/RentalVehicle>
 #include <KPublicTransport/Stopover>
 
 #include <QColor>
@@ -92,7 +91,7 @@ bool OpenTripPlannerParser::parseLocationFragment(const QJsonObject &obj, Locati
     if (!m_ifoptPrefix.isEmpty() && id.size() > m_ifoptPrefix.size() + 1 && id.startsWith(m_ifoptPrefix) && id.at(m_ifoptPrefix.size()) == QLatin1Char(':')) {
         const auto ifopt = QStringView(id).mid(m_ifoptPrefix.size() + 1);
         if (IfoptUtil::isValid(ifopt)) {
-            loc.setIdentifier(QStringLiteral("ifopt"), ifopt.toString());
+            loc.setIdentifier(IfoptUtil::identifierType(), ifopt.toString());
         }
     }
 
@@ -425,8 +424,12 @@ JourneySection OpenTripPlannerParser::parseJourneySection(const QJsonObject &obj
         section.setMode(JourneySection::PublicTransport);
         section.setRoute(detectAndParseRoute(obj));
     } else {
-        if (obj.value(QLatin1String("mode")).toString() == QLatin1String("BICYCLE")) {
-            section.setMode(JourneySection::RentedVehicle);
+        const auto mode = obj.value(QLatin1String("mode")).toString();
+        const auto isRented = obj.value(QLatin1String("rentedBike")).toBool();
+
+        if (mode.compare(QLatin1String("WALK"), Qt::CaseInsensitive) == 0 || mode.compare(QLatin1String("FOOT"), Qt::CaseInsensitive) == 0) {
+            section.setMode(JourneySection::Walking);
+        } else if (mode.compare(QLatin1String("BICYCLE"), Qt::CaseInsensitive) == 0) {
             RentalVehicle v;
             if (from.rentalVehicleStation().network().isValid()) {
                 v.setNetwork(from.rentalVehicleStation().network());
@@ -436,9 +439,18 @@ JourneySection OpenTripPlannerParser::parseJourneySection(const QJsonObject &obj
                 v.setNetwork(to.rentalVehicleStation().network());
             }
             v.setType(vehicleTypeFromTypes(v.network().vehicleTypes(), RentalVehicle::Bicycle));
-            section.setRentalVehicle(v);
+            if (v.network().isValid() || isRented) {
+                section.setMode(JourneySection::RentedVehicle);
+                section.setRentalVehicle(v);
+            } else {
+                section.setMode(JourneySection::IndividualTransport);
+                section.setIndividualTransport({ IndividualTransport::Bike });
+            }
+        } else if (mode.compare(QLatin1String("CAR"), Qt::CaseInsensitive) == 0) {
+            section.setMode(JourneySection::IndividualTransport);
+            section.setIndividualTransport({ IndividualTransport::Car });
         } else {
-            section.setMode(JourneySection::Walking);
+            qWarning() << "Unhandled OTP mode:" << mode;
         }
     }
 
@@ -465,12 +477,55 @@ JourneySection OpenTripPlannerParser::parseJourneySection(const QJsonObject &obj
     section.setIntermediateStops(std::move(stops));
 
     const auto geometryObj = obj.value(QLatin1String("legGeometry")).toObject();
+    QPolygonF poly;
     if (!geometryObj.empty()) {
-        QPolygonF poly;
         poly.reserve(geometryObj.value(QLatin1String("length")).toInt());
         const auto points = geometryObj.value(QLatin1String("points")).toString().toUtf8();
         PolylineDecoder<2> decoder(points.constData());
         decoder.readPolygon(poly);
+    }
+
+    const auto stepsArray = obj.value(QLatin1String("steps")).toArray();
+    if (!stepsArray.empty()) {
+        std::vector<PathSection> pathSections;
+        pathSections.reserve(stepsArray.size());
+        PathSection prevPathSec;
+        int prevPolyIdx = 0;
+        bool isFirstSection = true;
+        for (const auto &stepsV : stepsArray) {
+            const auto stepsObj = stepsV.toObject();
+            PathSection pathSec;
+            pathSec.setDescription(stepsObj.value(QLatin1String("legStepText")).toString());
+
+            if (!isFirstSection) {
+                const QPointF coord(stepsObj.value(QLatin1String("lon")).toDouble(), stepsObj.value(QLatin1String("lat")).toDouble());
+                const auto it = std::min_element(poly.begin() + prevPolyIdx, poly.end(), [coord](QPointF lhs, QPointF rhs) {
+                    return Location::distance(lhs.y(), lhs.x(), coord.y(), coord.x()) < Location::distance(rhs.y(), rhs.x(), coord.y(), coord.x());
+                });
+                int polyIdx = std::distance(poly.begin(), it);
+
+                QPolygonF subPoly;
+                subPoly.reserve(polyIdx - prevPolyIdx + 1);
+                std::copy(poly.begin() + prevPolyIdx, poly.begin() + polyIdx + 1, std::back_inserter(subPoly));
+                prevPathSec.setPath(std::move(subPoly));
+                prevPolyIdx = polyIdx;
+                pathSections.push_back(std::move(prevPathSec));
+            } else {
+                isFirstSection = false;
+            }
+            prevPathSec = pathSec;
+        }
+
+        QPolygonF subPoly;
+        subPoly.reserve(prevPolyIdx - poly.size() + 1);
+        std::copy(poly.begin() + prevPolyIdx, poly.end(), std::back_inserter(subPoly));
+        prevPathSec.setPath(std::move(subPoly));
+        pathSections.push_back(std::move(prevPathSec));
+
+        Path path;
+        path.setSections(std::move(pathSections));
+        section.setPath(std::move(path));
+    } else if (!poly.isEmpty()) {
         PathSection pathSec;
         pathSec.setPath(std::move(poly));
         Path path;
